Symmetric Key Encryption
========================

At a glance:

* **Cipher and Mode**: `AES-256-CTR`
* **Padding**: None (CTR mode doesn't pad)
* **Authentication**: `HMAC-SHA-256`
* **Construction**: `Encrypt then MAC`
* **Algorithm Backend**: `ext/openssl`

## `\Defuse\Crypto\Crypto`

**WARNING:** This encryption library is not a silver bullet. It only provides
symmetric encryption given a uniformly random key. This means you **MUST NOT**
use an ASCII string like a password as the key parameter, it **MUST** be
a uniformly random key generated by `createNewRandomKey()`. If you want to
encrypt something with a password, apply a password key derivation function
like PBKDF2 or scrypt with a random salt to generate a key.

***WARNING:*** Error handling is *very* important, especially for crypto code! 

How to use this code:
=====================

Generating a Key
----------------

```php
try {
    $key = \Defuse\Crypto\Crypto::createNewRandomKey();
    // WARNING: Do NOT encode $key with bin2hex() or base64_encode(),
    // they may leak the key to the attacker through side channels.
    // 
    // This library offers these two for cache-timing-safe encoding:
    //
    //     \Defuse\Crypto\Crypto\binToHex()
    //     \Defuse\Crypto\Crypto\hexToBin()
    //
} catch (\Defuse\Crypto\Exception\CryptoTestFailedException $ex) {
    die('Cannot safely create a key');
} catch (\Defuse\Crypto\Exception\CannotPerformOperationException $ex) {
    die('Cannot safely create a key');
}
```

Encrypting a Message
--------------------

```php
$message = 'ATTACK AT DAWN';
try {
    $ciphertext = \Defuse\Crypto\Crypto::Encrypt($message, $key);
} catch (\Defuse\Crypto\Exception\CryptoTestFailedException $ex) {
    die('Cannot safely perform encryption');
} catch (\Defuse\Crypto\Exception\CannotPerformOperationException $ex) {
    die('Cannot safely perform encryption');
}
```

Decrypting a Message
--------------------

```php
try {
    $decrypted = self::Decrypt($ciphertext, $key);
} catch (\Defuse\Crypto\Exception\InvalidCiphertextException $ex) { // VERY IMPORTANT
    // Either:
    //   1. The ciphertext was modified by the attacker,
    //   2. The key is wrong, or
    //   3. $ciphertext is not a valid ciphertext or was corrupted.
    // Assume the worst.
    die('DANGER! DANGER! The ciphertext has been tampered with!');
} catch (\Defuse\Crypto\Exception\CryptoTestFailedException $ex) {
    die('Cannot safely perform decryption');
} catch (\Defuse\Crypto\Exception\CannotPerformOperationException $ex) {
    die('Cannot safely perform decryption');
}
```



Decrypting a Message Encrypted with version 1 of this Library
-------------------------------------------------------------

```php
try {
    $decrypted = self::legacyDecrypt($ciphertext, $key);
} catch (\Defuse\Crypto\Exception\InvalidCiphertextException $ex) { // VERY IMPORTANT
    // Either:
    //   1. The ciphertext was modified by the attacker,
    //   2. The key is wrong, or
    //   3. $ciphertext is not a valid ciphertext or was corrupted.
    // Assume the worst.
    die('DANGER! DANGER! The ciphertext has been tampered with!');
} catch (\Defuse\Crypto\Exception\CryptoTestFailedException $ex) {
    die('Cannot safely perform decryption');
} catch (\Defuse\Crypto\Exception\CannotPerformOperationException $ex) {
    die('Cannot safely perform decryption');
}
```
